package net.horizoncode.nekorip;

import freemarker.template.Configuration;
import freemarker.template.TemplateExceptionHandler;
import lombok.Getter;
import net.horizoncode.nekorip.config.Config;
import net.horizoncode.nekorip.mysql.MySQLAPI;
import net.horizoncode.nekorip.template.BaseTemplateRoute;
import org.apache.commons.lang3.Validate;
import spark.Spark;

import java.io.File;

/**
 * Created by HorizonCode at 10/05/2021
 *
 * @author HorizonCode
 * @since 10/05/2021
 **/
@Getter
public class NekoRIP {

    @Getter
    private static NekoRIP instance;

    private final Configuration templates = new Configuration(Configuration.VERSION_2_3_27);

    private Config config;
    private MySQLAPI mySQL;
    private BaseTemplateRoute baseTemplateRoute;

    public static void main(String[] args) {
        new NekoRIP().init();
    }

    private void init() {
        Validate.isTrue(instance == null, "Instance is already initialized.");
        instance = this;

        File config = new File("config.toml");
        this.config = new Config(config);

        if (this.config.isJustCreated()) {
            System.out.println("No config found, created one. Please edit the config!");
            System.gc();
            System.exit(0);
            return;
        }
        System.out.println("Connecting to MySQL-Server...");
        String host = this.config.getStringOrDefault("mysql.host", "");
        int port = this.config.getIntOrDefault("mysql.port", 3306);
        String database = this.config.getStringOrDefault("mysql.database", "nekorip");
        String username = this.config.getStringOrDefault("mysql.username", "root");
        String password = this.config.getStringOrDefault("mysql.password", "");

        mySQL = MySQLAPI.getInstance();
        if (!mySQL.connect(host, port, username, password, database))
            System.out.println("Failed to connect to the MySQL-Server.");
        else
            System.out.println("Successfully connected to the MySQL-Server.");

        templates.setDefaultEncoding("UTF-8");
        templates.setTemplateExceptionHandler(TemplateExceptionHandler.RETHROW_HANDLER);
        templates.setLogTemplateExceptions(false);
        templates.setWrapUncheckedExceptions(true);
        templates.setTagSyntax(Configuration.SQUARE_BRACKET_TAG_SYNTAX);

        String web_host = this.config.getStringOrDefault("web.host", "0.0.0.0");
        int web_port = this.config.getIntOrDefault("web.port", 8000);

        Spark.ipAddress(web_host);
        Spark.port(web_port);
        Spark.staticFileLocation("/static/");

        System.out.println("Setting up Routes...");
        Spark.get("/", baseTemplateRoute = new BaseTemplateRoute("home.html"));

        System.out.printf("Listening on %s:%d...%n", web_host, web_port);

    }

}
