package net.horizoncode.nekorip.config;

import lombok.AccessLevel;
import lombok.Getter;
import net.horizoncode.nekorip.NekoRIP;
import org.apache.commons.io.FileUtils;
import org.tomlj.Toml;
import org.tomlj.TomlParseResult;

import java.io.File;
import java.io.IOException;

@Getter
public class Config {

    @Getter(AccessLevel.NONE)
    private final File configFile;

    @Getter(AccessLevel.PACKAGE)
    private TomlParseResult toml;

    private boolean justCreated;


    public Config(File cfg) {
        configFile = cfg;
        if (!configFile.exists()) {
            try {
                FileUtils.copyInputStreamToFile(NekoRIP.class.getResourceAsStream("/" + cfg.getName()), configFile);
                justCreated = true;
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else {
            if (configFile.isDirectory()) {
                try {
                    FileUtils.copyInputStreamToFile(NekoRIP.class.getResourceAsStream("/" + cfg.getName()), configFile);
                    justCreated = true;
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        if (justCreated)
            return;

        try {
            toml = Toml.parse(configFile.toPath());
            if (toml.hasErrors()) {
                toml.errors().forEach(error -> System.err.println(error.toString()));
                System.exit(-1);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public int getIntOrDefault(String key, int defaultValue) {
        return getToml().contains(key) && getToml().get(key) instanceof Integer ? (int) getToml().get(key) : defaultValue;
    }

    public boolean getBooleanOrDefault(String key, boolean defaultValue) {
        return getToml().contains(key) && getToml().get(key) instanceof Boolean ? getToml().getBoolean(key) : defaultValue;
    }

    public String getStringOrDefault(String key, String defaultValue) {
        return getToml().contains(key) ? getToml().getString(key) : defaultValue;
    }

}
