package net.horizoncode.nekorip.mysql;


import java.sql.*;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

public class MySQLAPI {
    private static final MySQLAPI INSTANCE;

    static {
        INSTANCE = new MySQLAPI();
    }

    private String host;
    private int port;
    private String user;
    private String password;
    private String database;
    private Connection connection;

    private MySQLAPI() {
    }

    public static MySQLAPI getInstance() {
        return MySQLAPI.INSTANCE;
    }

    public boolean connect(final String host, final int port, final String user, final String password, final String database) {
        this.host = host;
        this.port = port;
        this.user = user;
        this.password = password;
        this.database = database;
        return this.openConnection();
    }

    private boolean openConnection() {
        try {
            if (this.connection != null) {
                this.connection.close();
            }
            Class.forName("com.mysql.cj.jdbc.Driver");
            this.connection = DriverManager.getConnection("jdbc:mysql://" + this.host + ":" + this.port + "/" + this.database, this.user, this.password);
            establishConnection();
        } catch (Exception e) {
            System.out.println("MySQL error while trying connecting to " + this.host + ":" + this.port + "/" + this.database + ": " + e.getMessage());
            return false;
        }
        System.out.println("MySQL connected to " + this.host + ":" + this.port + "/" + this.database);
        return true;
    }

    private void establishConnection() {
        ScheduledExecutorService scheduleTaskExecutor = Executors.newScheduledThreadPool(5);

        scheduleTaskExecutor.scheduleAtFixedRate(() -> {
            try {
                Statement s = getConnection().createStatement();
                s.executeQuery("SELECT 1+1");
                s.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }, 0, 10, TimeUnit.MINUTES);
    }

    public void closeConnection() {
        try {
            System.out.println("MySQL connection closed!");
            connection.close();
        } catch (SQLException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    public void closeRessources(final ResultSet rs, final PreparedStatement st, final Statement s) {
        if (rs != null) {
            try {
                rs.close();
            } catch (SQLException ex) {
            }
        }
        if (st != null) {
            try {
                st.close();
            } catch (SQLException ex2) {
            }
        }
        if (s != null) {
            try {
                s.close();
            } catch (SQLException ex2) {
            }
        }
    }

    public void closeRessources(final ResultSet rs) {
        this.closeRessources(rs, null, null);
    }

    public void closeRessources(final PreparedStatement st) {
        this.closeRessources(null, st, null);
    }

    public void closeRessources(final Statement s) {
        this.closeRessources(null, null, s);
    }

    public Connection getConnection() {
        return connection;
    }

}
